@@ -1,6 +1,6 @@
-import random
 from http.client import HTTPConnection, HTTPSConnection
 from socket import SocketIO
+from urllib.error import HTTPError
 from urllib.request import urlopen
 from unittest import mock
 
@@ -42,6 +42,50 @@ def test_crumb_capture(sentry_init, capture_events):
     )
 
 
+@pytest.mark.parametrize(
+    "status_code,level",
+    [
+        (200, None),
+        (301, None),
+        (403, "warning"),
+        (405, "warning"),
+        (500, "error"),
+    ],
+)
+def test_crumb_capture_client_error(sentry_init, capture_events, status_code, level):
+    sentry_init(integrations=[StdlibIntegration()])
+    events = capture_events()
+
+    url = f"http://localhost:{PORT}/status/{status_code}"  # noqa:E231
+    try:
+        urlopen(url)
+    except HTTPError:
+        pass
+
+    capture_message("Testing!")
+
+    (event,) = events
+    (crumb,) = event["breadcrumbs"]["values"]
+
+    assert crumb["type"] == "http"
+    assert crumb["category"] == "httplib"
+
+    if level is None:
+        assert "level" not in crumb
+    else:
+        assert crumb["level"] == level
+
+    assert crumb["data"] == ApproxDict(
+        {
+            "url": url,
+            SPANDATA.HTTP_METHOD: "GET",
+            SPANDATA.HTTP_STATUS_CODE: status_code,
+            SPANDATA.HTTP_FRAGMENT: "",
+            SPANDATA.HTTP_QUERY: "",
+        }
+    )
+
+
 def test_crumb_capture_hint(sentry_init, capture_events):
     def before_breadcrumb(crumb, hint):
         crumb["data"]["extra"] = "foo"
@@ -140,12 +184,13 @@ def test_outgoing_trace_headers(sentry_init, monkeypatch):
 
     sentry_init(traces_sample_rate=1.0)
 
-    headers = {}
-    headers["baggage"] = (
-        "other-vendor-value-1=foo;bar;baz, sentry-trace_id=771a43a4192642f0b136d5159a501700, "
-        "sentry-public_key=49d0f7386ad645858ae85020e393bef3, sentry-sample_rate=0.01337, "
-        "sentry-user_id=Am%C3%A9lie, other-vendor-value-2=foo;bar;"
-    )
+    headers = {
+        "baggage": (
+            "other-vendor-value-1=foo;bar;baz, sentry-trace_id=771a43a4192642f0b136d5159a501700, "
+            "sentry-public_key=49d0f7386ad645858ae85020e393bef3, sentry-sample_rate=0.01337, "
+            "sentry-user_id=Am%C3%A9lie, sentry-sample_rand=0.132521102938283, other-vendor-value-2=foo;bar;"
+        ),
+    }
 
     transaction = Transaction.continue_from_headers(headers)
 
@@ -175,8 +220,9 @@ def test_outgoing_trace_headers(sentry_init, monkeypatch):
         expected_outgoing_baggage = (
             "sentry-trace_id=771a43a4192642f0b136d5159a501700,"
             "sentry-public_key=49d0f7386ad645858ae85020e393bef3,"
-            "sentry-sample_rate=0.01337,"
-            "sentry-user_id=Am%C3%A9lie"
+            "sentry-sample_rate=1.0,"
+            "sentry-user_id=Am%C3%A9lie,"
+            "sentry-sample_rand=0.132521102938283"
         )
 
         assert request_headers["baggage"] == expected_outgoing_baggage
@@ -189,11 +235,9 @@ def test_outgoing_trace_headers_head_sdk(sentry_init, monkeypatch):
     mock_send = mock.Mock()
     monkeypatch.setattr(HTTPSConnection, "send", mock_send)
 
-    # make sure transaction is always sampled
-    monkeypatch.setattr(random, "random", lambda: 0.1)
-
     sentry_init(traces_sample_rate=0.5, release="foo")
-    transaction = Transaction.continue_from_headers({})
+    with mock.patch("sentry_sdk.tracing_utils.Random.uniform", return_value=0.25):
+        transaction = Transaction.continue_from_headers({})
 
     with start_transaction(transaction=transaction, name="Head SDK tx") as transaction:
         HTTPSConnection("www.squirrelchasers.com").request("GET", "/top-chasers")
@@ -215,6 +259,7 @@ def test_outgoing_trace_headers_head_sdk(sentry_init, monkeypatch):
 
         expected_outgoing_baggage = (
             "sentry-trace_id=%s,"
+            "sentry-sample_rand=0.250000,"
             "sentry-environment=production,"
             "sentry-release=foo,"
             "sentry-sample_rate=0.5,"
@@ -334,7 +379,7 @@ def test_span_origin(sentry_init, capture_events):
     events = capture_events()
 
     with start_transaction(name="foo"):
-        conn = HTTPSConnection("example.com")
+        conn = HTTPConnection("example.com")
         conn.request("GET", "/foo")
         conn.getresponse()
 
@@ -353,25 +398,16 @@ def test_http_timeout(monkeypatch, sentry_init, capture_envelopes):
 
     envelopes = capture_envelopes()
 
-    with start_transaction(op="op", name="name"):
-        try:
-            conn = HTTPSConnection("www.squirrelchasers.com")
-            conn.request("GET", "/top-chasers")
+    with pytest.raises(TimeoutError):
+        with start_transaction(op="op", name="name"):
+            conn = HTTPSConnection("www.example.com")
+            conn.request("GET", "/bla")
             conn.getresponse()
-        except Exception:
-            pass
 
-    items = [
-        item
-        for envelope in envelopes
-        for item in envelope.items
-        if item.type == "transaction"
-    ]
-    assert len(items) == 1
-
-    transaction = items[0].payload.json
+    (transaction_envelope,) = envelopes
+    transaction = transaction_envelope.get_transaction_event()
     assert len(transaction["spans"]) == 1
 
     span = transaction["spans"][0]
     assert span["op"] == "http.client"
-    assert span["description"] == "GET https://www.squirrelchasers.com/top-chasers"
+    assert span["description"] == "GET https://www.example.com/bla"
